@@ -10,11 +10,14 @@ employed are:
   * HTTP header fingerprinting (e.g. REMOTE_ADDR, HTTP_USER_AGENT)
   * per-request nonces (with configurable timeout and duplicate window)
   * periodic cycling of session keys
+  * maintaining a second key for secure connections
+  * marking session keys as "HttpOnly"
 
 As always, there's a tradeoff here - these security measures involve increased
-processing per request and more frequent writes to the session store.  But by
-adjusting the various settings offered by this module, you should be able to
-find a compromise that's suitable for your project.
+processing per request and more frequent writes to the session store.  You also
+run the risk of terminating valid sessions that happen to look suspicious.
+But by adjusting the various settings offered by this module, you should be
+able to find a compromise that's suitable for your project.
 
 To use this module, simply include it in MIDDLEWARE_CLASSES after the standard
 session middleware:
@@ -63,7 +66,7 @@ The following settings are available to tweak the behaviour of this module:
           store.  Setting it to zero means a new nonce is generated on every
           request.  Setting it to None will disable the use of nonces entirely.
 
-          Default:  0
+          Default:  1
 
 
   PSESSION_NONCE_WINDOW:  Number of nonces prior to the current nonce that will
@@ -84,7 +87,7 @@ The following settings are available to tweak the behaviour of this module:
           necessary if users will perform multiple overlapping requests.
           Setting it to None means the window never times out.
 
-          Default:  0.5
+          Default:  1
 
 
   PSESSION_KEY_TIMEOUT:  Time (in seconds) after which the session key will
@@ -106,6 +109,15 @@ The following settings are available to tweak the behaviour of this module:
           Default:  "sessionnonce"
 
 
+  PSESSION_SECURE_COOKIE_NAME:  Name of cookie used for the additional "secure
+          connections only" session key.  This additional information is only
+          passed between the server and client over a secure connection; if
+          a sniffer or man-in-the-middle attack compromises all HTTP traffic,
+          they still cannot forge session requests over HTTPS.
+
+          Default:  "sessionid_https"
+
+
   PSESSION_CLEAR_SESSION_FUNCTION:  Function called to clear the session if a
           potential attack is detected.  An importable name or callable object
           may be given, taking a request object as its only argument.
@@ -132,7 +144,7 @@ being terminated.  Consider some of the following adjustments:
 
 """
 
-__version__ = "0.1.0"
+__version__ = "0.2.0"
 
 import time
 
@@ -148,17 +160,21 @@ if not hasattr(settings,"PSESSION_CHECK_HEADERS"):
     check_headers = ("REMOTE_ADDR","HTTP_X_FORWARDED_FOR","HTTP_USER_AGENT",)
     settings.PSESSION_CHECK_HEADERS = check_headers
 if not hasattr(settings,"PSESSION_NONCE_TIMEOUT"):
-    settings.PSESSION_NONCE_TIMEOUT = 0
+    settings.PSESSION_NONCE_TIMEOUT = 1
 if not hasattr(settings,"PSESSION_NONCE_WINDOW"):
     settings.PSESSION_NONCE_WINDOW = 1
 if not hasattr(settings,"PSESSION_NONCE_WINDOW_TIMEOUT"):
-    settings.PSESSION_NONCE_WINDOW_TIMEOUT = 0.5
+    settings.PSESSION_NONCE_WINDOW_TIMEOUT = 1
 if not hasattr(settings,"PSESSION_KEY_TIMEOUT"):
     settings.PSESSION_KEY_TIMEOUT = None
 if not hasattr(settings,"PSESSION_SESSION_KEY"):
     settings.PSESSION_SESSION_KEY = "PARANOID_SESSION_DATA"
 if not hasattr(settings,"PSESSION_COOKIE_NAME"):
     settings.PSESSION_COOKIE_NAME = "sessionnonce"
+if not hasattr(settings,"PSESSION_SECURE_COOKIE_NAME"):
+    settings.PSESSION_SECURE_COOKIE_NAME = "sessionid_https"
+if not hasattr(settings,"PSESSION_COOKIE_HTTPONLY"):
+    settings.PSESSION_COOKIE_HTTPONLY = True
 if hasattr(settings,"PSESSION_REQUEST_FILTER_FUNCTION"):
     request_filter = get_callable(settings.PSESSION_REQUEST_FILTER_FUNCTION)
     settings.PSESSION_REQUEST_FILTER_FUNCTION = request_filter
@@ -225,6 +241,7 @@ class SessionFingerprint(object):
         self.last_nonce_time = 0
         self.last_key_time = now
         self.last_request_time = now
+        self.secure_key = None
 
     def check_request(self,request):
         """Check that the given request is valid for this session.
@@ -233,11 +250,21 @@ class SessionFingerprint(object):
         if the duplicate nonce window is enabled and it matches a sufficiently
         recent hash.
         """
+        if request.is_secure():
+            if self.secure_key is None:
+                seed = (randrange(0,MAX_NONCE_SEED),settings.SECRET_KEY)
+                self.secure_key = md5_constructor("%s%s" % seed).hexdigest()
+                request.session.modified = True
+            else:
+                cookie_name = settings.PSESSION_SECURE_COOKIE_NAME
+                key = request.COOKIES.get(cookie_name,"")
+                if key != self.secure_key:
+                    return False
         hash = self.request_hash(request)
         if hash != self.hash:
             return False
         if settings.PSESSION_NONCE_TIMEOUT is not None:
-            nonce = request.COOKIES[settings.PSESSION_COOKIE_NAME]
+            nonce = request.COOKIES.get(settings.PSESSION_COOKIE_NAME,"")
             if nonce not in self.get_valid_nonces():
                 return False
         self.last_request_time = time.time()
@@ -270,6 +297,20 @@ class SessionFingerprint(object):
                 request.session.cycle_key()
                 self.last_key_time = now
                 request.session.modified = True
+        #  Send the secure_key if the client doesn't have it yet, or has
+        #  an incorrect value.  Remember, this will only be called for
+        #  valid requests!
+        if request.is_secure():
+            cookie_name = settings.PSESSION_SECURE_COOKIE_NAME
+            if self.secure_key != request.COOKIES.get(cookie_name,""):
+                self.set_secure_key_cookie(request,response)
+        #  Force the session cookie to be HttpOnly.
+        #  This works even though we get called before the session cookie is
+        #  sent; fortunately SimpleCookie remembers individual settings even
+        #  if you re-assign the cookie.
+        if request.session.modified or settings.SESSION_SAVE_EVERY_REQUEST:
+            key = request.session.session_key
+            self._set_cookie(request,response,settings.SESSION_COOKIE_NAME,key)
             
     def request_hash(self,request):
         """Create a hash of the given request's fingerprint data.
@@ -304,12 +345,22 @@ class SessionFingerprint(object):
         yield nonces.next()
 
     def set_nonce_cookie(self,request,response):
-        """Set the nonce cookie on the given response.
+        """Set the nonce cookie on the given response."""
+        nonce = list(self.get_valid_nonces())[-2]
+        self._set_cookie(request,response,settings.PSESSION_COOKIE_NAME,nonce)
+
+    def set_secure_key_cookie(self,request,response):
+        """Set the secure-key cookie on the given response."""
+        name = settings.PSESSION_SECURE_COOKIE_NAME
+        self._set_cookie(request,response,name,self.secure_key,True)
+
+    def _set_cookie(self,request,response,name,value,secure=False):
+        """Set a session-related cookie.
 
         This duplicates the cookie-setting logic in the session middleware,
         so it will expire under the same rules as the session itself.
         """
-        nonce = list(self.get_valid_nonces())[-2]
+        secure = secure or settings.SESSION_COOKIE_SECURE
         if request.session.get_expire_at_browser_close():
             max_age = None
             expires = None
@@ -317,11 +368,13 @@ class SessionFingerprint(object):
             max_age = request.session.get_expiry_age()
             expires_time = time.time() + max_age
             expires = cookie_date(expires_time)
-        response.set_cookie(settings.PSESSION_COOKIE_NAME,nonce,
+        response.set_cookie(name,value,secure=secure,
                             max_age=max_age,expires=expires,
                             domain=settings.SESSION_COOKIE_DOMAIN,
-                            path=settings.SESSION_COOKIE_PATH,
-                            secure=settings.SESSION_COOKIE_SECURE or None)
+                            path=settings.SESSION_COOKIE_PATH)
+        if settings.PSESSION_COOKIE_HTTPONLY:
+            if "httponly" in response.cookies[name]:
+                response.cookies[name]["httponly"] = True
 
     def refresh_from_session(self,request):
         """Refresh internal data from the session store."""
